
#import <Foundation/Foundation.h>
#import "iCore.h"

@class B4ITableCell;

/**
* TableView shows a list of items (cells).
*The TableView is divided into one or more sections.
*TableView reuses the cells views and only updates the content. TableView can show very large lists.
*<b>TableView2 which adds support for action buttons, requires iOS 8+</b>.
 *
*/
//~shortname: TableView
//~ObjectWrapper: UITableView*
//~version: 1.00
//~event: SelectedChanged (SectionIndex As Int, Cell As TableCell)
//~event: AccessoryButtonClicked (SectionIndex As Int, Cell As TableCell)
//~event: ActionButtonClicked (SectionIndex As Int, Cell As TableCell, Text As String)
//~DesignerProperty: Key: Grouped, DisplayName: Grouped, FieldType: Boolean, DefaultValue: False
@interface B4ITableView : B4IViewWrapper<B4IDesignerCustomView>
//Returns the number of sections added to the TableView.
@property (nonatomic, readonly)int NumberOfSections;
//Returns the selected row item. Return Null if there is no selected row.
@property (nonatomic, readonly)B4ITableCell *SelectedRow;
//Returns the index of the section that holds the selected item. Returns -1 if there is no selected item.
@property (nonatomic, readonly)int SelectedSection;
//Gets or sets the separator color.
@property (nonatomic) int SeparatorColor;
//Gets or sets the rows heights.
@property (nonatomic)float RowHeight;
@property (nonatomic, readonly)int SCROLL_TOP;
@property (nonatomic, readonly)int SCROLL_MIDDLE;
@property (nonatomic, readonly)int SCROLL_BOTTOM;
/**
 * When the TableView is added as a custom view (from the designer) it is added to a base panel.
 *This property will return the base panel. You can use this property to move the TableView.
 *Note that if the view was added by code then this property will return the TableView itself.
 */
@property (nonatomic, readonly)B4IViewWrapper *BaseView;

/**
* Returns a List with the TableCells in the specified index.
*Modifying this list will affect the items in this section. You should call TableView.ReloadSection if you have added or removed items.
*/
- (B4IList *)GetItems:(int)SectionIndex;

- (void)DesignerCreateView:(B4IPanelWrapper *)base :(B4ILabelWrapper *)lw :(B4IMap *)props;

/**
* Initializes the TableView.
*EventName - Set the subs that will handle the events.
*Grouped - Whether to use the grouped style or the plain style. The plain style uses floating headers.
*/
- (void)Initialize:(B4I *)bi :(NSString *)EventName :(BOOL)Grouped;

/**
* Adds a section to the TableView with the given header title and footer title.
*/
- (void)AddSection:(NSString *)Header :(NSString *)Footer;

/**
* Adds a single line item to the TableView. This method returns a TableCell object which you can further modify.
*/
- (B4ITableCell *)AddSingleLine:(NSString *)Text;

/**
* Adds a two lines item to the TableView. This method returns a TableCell object which you can further modify.
*/
- (B4ITableCell *)AddTwoLines:(NSString *)Text :(NSString *)DetailText;

/**
* Sets the selected item and scrolls to this item.
*SectionIndex - The item section index (o if there is only a single section).
*RowIndex - The index of the item in the section. Pass -1 to deselect the current selected item.
*/
- (void)SetSelection:(int)SectionIndex :(int)RowIndex;
//Removes all items from the TableView.
- (void)Clear;
//Reloads all the TableCell data.
- (void)ReloadAll;
//Reloads the data of a specific section. You should call this method if you have added or removed items from this section.
- (void)ReloadSection:(int)SectionIndex;
//Call this method before you call InsertCells or RemoveCells. You must call EndUpdates at the end.
- (void)BeginUpdates;

/**
* This method should only be called after BeginUpdates was called.
*Inserts one or more cells in the list. You do not need to call ReloadSection to apply the changes.
*SectionIndex - The section index.
*RowIndex - The first item will be added at this index.
*Cells - A list with TableCells.
*/
- (void)InsertCells:(int)SectionIndex :(int)RowIndex :(B4IList *)Cells;

/**
* This method should only be called after BeginUpdates was called.
*Removes one or more cells from list. You do not need to call ReloadSection to apply the changes.
*SectionIndex - The section index.
*StartRowIndex - Index of the first item that will be removed.
*Count - Number of items to remove.
*/
- (void)RemoveCells:(int)SectionIndex :(int)StartRowIndex :(int)Count;

/**
* Ends an updates block that was previously started with BeginUpdates.
*/
- (void)EndUpdates;

/**
* Scrolls the table to the given section and row index.
*ScrollPosition - One of the SCROLL constants. This item will be scrolled to this position.
*/
- (void)ScrollTo:(int)SectionIndex :(int)RowIndex :(int)ScrollPosition;
@end

/**
* Represents a single item in a TableView.
*/
//~shortname: TableCell
@interface B4ITableCell : NSObject
//No accessory view.
@property (nonatomic, readonly)int ACCESSORY_NONE;
//Shows a chevron symbol (arrow). This accessory symbol does not track touch events.
@property (nonatomic, readonly)int ACCESSORY_INDICATOR;
//Shows a chevron image with an info button. This symbol does track touch events.
@property (nonatomic, readonly)int ACCESSORY_DISCLOSURE_BUTTON;
//Shows an info button. This symbol does track touch events.
@property (nonatomic, readonly)int ACCESSORY_DETAIL_BUTTON;
//Shows a check mark. This symbol does not track touch events.
@property (nonatomic, readonly)int ACCESSORY_CHECKMARK;
/**
* The Panel together with its child views will be added to the cell.
*Usually you will also need to set the table RowHeight and set the Panel's width and height.
*/
@property (nonatomic)B4IPanelWrapper *CustomView;
//~hide
@property (nonatomic)NSString *reusableIdentifier;
//~hide
@property (nonatomic, weak)UITableViewCell *nativeCell;
//~hide
@property (nonatomic)UITableViewCellStyle cellStyle;
//~hide
@property (nonatomic)NSMutableArray *actionButtons;
//Gets or sets the main text attributed string.
@property (nonatomic)B4IAttributedString *Text;
//Gets or sets the second line attributed string.
@property (nonatomic)B4IAttributedString *DetailText;
//Gets or sets the item bitmap.
@property (nonatomic)B4IBitmap *Bitmap;
//Gets or sets a tag object that is tied to this cell.
@property (nonatomic)NSObject *Tag;
//Gets or sets the indentation width. Default value is 10 pixels (points).
@property (nonatomic)float IndentationWidth;
//Gets or sets the indentation level. Default value is 0.
@property (nonatomic)int IndentationLevel;
//Gets or sets the accessory symbol that shows at the right side of the cell.
@property (nonatomic)int AccessoryType;
//Gets or sets whether the item will be highlighted when selected. Default value is True.
@property (nonatomic)BOOL ShowSelection;
//Initializes a single line item with the given text.
- (void)InitializeSingleLine:(NSString *)Text;
//Initializes a two lines item with the given texts.
- (void)InitializeTwoLines:(NSString *)Text :(NSString *)DetailText;

//~hide
- (UITableViewCell *)createUI:(UITableViewCell *)current;
/**
 *Adds an action button. The button will be displayed when the user swipes the item.
 *The TableView_ActionButtonClicked event will be raised when the button is clicked.
 */
- (void)AddActionButton:(NSString *)Text :(int)Color;
/**
 * Removes the action buttons from the button.
 */
- (void)ClearActionButtons;

/**
* Applies any changes done to visible cell. You should call this method after you update one or more properties
*of an existing cell.
*/
- (void)Update;
@end

//~shortname:NavControlExtra
//~ObjectWrapper: UINavigationController*
@interface B4INavControlExtra: B4IObjectWrapper
//Gets or sets whether tapping on the page will hide or show the bars.
@property (nonatomic) BOOL HidesBarsOnTap;
//Gets or sets whether a swipe gesture will hide or show the bars.
@property (nonatomic) BOOL HidesBarOnSwipe;
//Gets or sets whether the bars should be hidden in a vertically compact environment (landscape on iPhone for example).
@property (nonatomic) BOOL HidesBarsWhenVerticallyCompact;
//Gets or sets whether the bars should be hidden when the keyboard appears.
@property (nonatomic) BOOL HidesBarsWhenKeyboardAppears;


- (void)Initialize:(B4INavigationControllerWrapper *)NavigationController;
@end
/**
 * Improved version of WebView. It is based on the native WKWebView instead of UIWebView.
 * This is a custom view. It should be added from the designer.
 * OverrideUrl event allows you to cancel requests. Return True to cancel (override) the request.
 */
//~shortname:WKWebView
//~ObjectWrapper: WKWebView*
//~Event:PageFinished (Success As Boolean, Url As String)
//~Event:OverrideUrl (Url As String) As Boolean
//~Event:JSComplete (Success As Boolean, Tag As Object, Result As String)
//~DesignerProperty: Key: SuppressRendering, DisplayName: Suppress Rendering, FieldType: Boolean, DefaultValue: False, Description: Suppress rendering until content is ready.
@interface B4IWKWebvViewWrapper: B4IObjectWrapper <B4IDesignerCustomView>
/**
 * Returns the base panel. The WebView is added to this panel.
 */
@property (nonatomic, readonly)B4IViewWrapper *BaseView;
/**
 * Returns the current page title.
 */
@property (nonatomic, readonly)NSString *Title;
/**
 * Returns the estimated loading progress. The returned value is between 0 to 1.
 */
@property (nonatomic, readonly)double EstimatedProgress;
- (void)DesignerCreateView:(B4IPanelWrapper *)base :(B4ILabelWrapper *)lw :(B4IMap *)props;
/**
* Loads a url. Note that you cannot use this method to load files from the Assets folder.
* Example:<code>
* WebView1.LoadUrl("http://www.google.com")</code>
*/
- (void)LoadUrl:(NSString *)Url;
/**
* Loads the given html code. The path is relative to the assets folder.
*/
- (void)LoadHtml:(NSString *)Html;

//Returns true if the WebView can navigate to the previous page.
- (BOOL)CanGoBack;
//Returns true if the WebView can navigate forward.
- (BOOL)CanGoForward;
//Navigates to the previous page.
- (void)GoBack;
//Navigates to the next page in the stack of pages.
- (void)GoForward;
//Stops loading the current page.
- (void)StopLoading;
//Clear caches and reloads the current page.
- (void)Reload;
/**
 * Asynchronously evaluates the JavaScript. The JSComplete event will be raised.
 * Tag - An object that will be included in the event.
 * Script - The JavaScript to evaluate.
 */
- (void)EvaluateJavaScript:(NSObject *)Tag :(NSString *)Script;
@end

/**
 * A simple view that adds a blur effect to the views underneath it.
 */
//~shortname: VisualEffectView
//~objectwrapper: VisualEffectView*
//~DesignerProperty: Key: style, DisplayName: Effect Style, FieldType: String, List: Dark|ExtraLight|Light, DefaultValue: Light
@interface B4IVisualEffectView : B4IObjectWrapper<B4IDesignerCustomView>
/**
 * Returns the base panel. The "effect view" is added to this panel.
 */
@property (nonatomic, readonly)B4IViewWrapper *BaseView;
- (void)DesignerCreateView:(B4IPanelWrapper *)base :(B4ILabelWrapper *)lw :(B4IMap *)props;
@end